﻿using Telerik.Windows.Controls;
using Telerik.Windows.Data;
using Telerik.Windows.Examples;
using System.Collections.Generic;
using System.Linq;
using System;
using System.ComponentModel;

namespace NewDescriptorsWPF
{
	public class MainViewModel
	{
		private Northwind dataSource;
		private QueryableCollectionView employees;

		public MainViewModel()
		{
			this.ToggleFilterCommand = new DelegateCommand(this.ToggleFilter);
			this.ToggleSortCommand = new DelegateCommand(this.ToggleSort);
			this.ToggleGroupCommand = new DelegateCommand(this.ToggleGroup);
		}

		public Northwind DataSource
		{
			get
			{
				if (this.dataSource == null)
				{
					this.dataSource = new Northwind();
				}

				return this.dataSource;
			}
		}

		public QueryableCollectionView Employees
		{
			get
			{
				if (this.employees == null)
				{
					this.employees = new QueryableCollectionView(this.DataSource.EmployeesCollection);
				}

				return this.employees;
			}
		}

		public DelegateCommand ToggleFilterCommand { get; private set; }

		public DelegateCommand ToggleSortCommand { get; private set; }

		public DelegateCommand ToggleGroupCommand { get; private set; }

		private void ToggleFilter(object parameter)
		{
			if (this.Employees.FilterDescriptors.Count == 0)
			{
				var prospects = this.GetPromotionProspects();

				var descriptor = new FilterDescriptor<Employee> { FilteringExpression = e => prospects.Contains(e) };
				this.Employees.FilterDescriptors.Add(descriptor);
			}
			else
			{
				this.Employees.FilterDescriptors.Clear();
			}
		}

		private void ToggleSort(object parameter)
		{
			if (this.Employees.SortDescriptors.Count == 0)
			{
				var descriptor = new SortDescriptor<Employee, double>
				{
					SortingExpression = e => e.Orders.SelectMany(o => o.Details).Sum(d => d.UnitPrice * d.Quantity),
					SortDirection = ListSortDirection.Descending
				};

				this.Employees.SortDescriptors.Add(descriptor);
			}
			else
			{
				this.Employees.SortDescriptors.Clear();
			}
		}

		private void ToggleGroup(object parameter)
		{
			if (this.Employees.GroupDescriptors.Count == 0)
			{
				using (this.Employees.DeferRefresh())
				{
					var descriptor = new GroupDescriptor<Employee, int, int>
					{
						GroupingExpression = e => e.Orders.Where(o => o.Details.Any(d => d.Product.ProductName.Contains("Syrup"))).Count(),
						SortDirection = ListSortDirection.Ascending
					};

					this.Employees.GroupDescriptors.Add(descriptor); 
				}
			}
			else
			{
				using (this.Employees.DeferRefresh())
				{
					this.Employees.GroupDescriptors.Clear();
				}
			}
		}

		private IEnumerable<Employee> GetPromotionProspects()
		{
			yield return this.DataSource.EmployeesCollection.Where(e => e.FirstName == "Nancy").FirstOrDefault();

			yield return this.DataSource.EmployeesCollection.Where(e => e.FirstName == "Steven").FirstOrDefault();

			yield return this.DataSource.EmployeesCollection.Where(e => e.FirstName == "Laura").FirstOrDefault();
		}
	}
}
